#include <assert.h>
#include <signal.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>

#include "floppy_c.h"

FILE *fifo_in, *fifo_out;

void my_fprintf(FILE *file, const char *format, ...) {
    va_list args;
    va_start(args, format);
    if (vfprintf(file, format, args) < 0) {
        exit(0);
    }
    va_end(args);
}

void my_fscanf(FILE *file, int expected, const char *format, ...) {
    va_list args;
    va_start(args, format);
    if (vfscanf(file, format, args) != expected) {
        exit(0);
    }
    va_end(args);
}

void run1() {
    int subtask_id, N;
    // Receive subtask_id, N.
    my_fscanf(fifo_in, 2, "%d%d", &subtask_id, &N);
    // Receive v.
    int v[N];
    for (int i = 0; i < N; ++i) {
        my_fscanf(fifo_in, 1, "%d", &v[i]);
    }
    read_array(subtask_id, N, v);
}

void save_to_floppy(int L, char* bits) {
    // Save bits to floppy disk.
    my_fprintf(fifo_out, "%d\n", L);
    for (int i = 0; i < L; ++i) {
        my_fprintf(fifo_out, "%c", bits[i]);
    }
    fflush(fifo_out);
    exit(0);
}

void run2() {
    int subtask_id, N;
    my_fscanf(fifo_in, 2, "%d%d", &subtask_id, &N);

    // Receive bits from floppy disk.
    int L;
    my_fscanf(fifo_in, 1, "%d", &L);
    char floppy[L + 1];
    my_fscanf(fifo_in, 0, "\n");
    for (int i = 0; i < L; i++) {
      my_fscanf(fifo_in, 1, "%c", &floppy[i]);
    }
    floppy[L] = '\0';

    // Receive queries.
    int M;
    my_fscanf(fifo_in, 1, "%d", &M);
    int a[M], b[M];
    for (int i = 0; i < M; ++i) {
        my_fscanf(fifo_in, 2, "%d%d", &a[i], &b[i]);
    }

    int* query_answers = solve_queries(subtask_id, N, L, floppy, M, a, b);

    // Send query answers.
    for (int i = 0; i < M; ++i) {
        my_fprintf(fifo_out, "%d ", query_answers[i]);
    }
    fflush(fifo_out);
    free(query_answers);
}

int main(int argc, char** argv) {
    signal(SIGPIPE, SIG_IGN);

    if (argc != 4) {
        fprintf(stderr, "Invalid number of parameters to stub.");
    }
    fifo_in = fopen(argv[2], "r");
    fifo_out = fopen(argv[1], "w");
    int run = atoi(argv[3]);
    if (run == 0) {
        // Run 1.
        run1();
    } else if (run == 1) {
        // Run 2.
        run2();
    } else {
        fprintf(stderr, "Invalid run index for stub.");
    }
    return 0;
}
